`timescale 1ns / 1ps

//////////////////////////////////////////////////////////////////////////////////
// Company: TeamANIMALS
// Engineer: Leif Jentoft
// 
// Create Date:   17:07:57 12/07/2007 
// Design Name:	Animal 	
// Module Name:   animal 
// Project Name:  TeamANIMALS FPGA Cat & Mouse Game
// Target Devices: Spartan S3 xc3s2200ft256
// Description:
//		Transform from control to graphics
//
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

module human_input(delta_heading, button_left, button_right, gameclock);
	input button_left, button_right, gameclock;
	output delta_heading;
	
	reg [3:0] delta_heading;
	reg buttons_were_pressed;

	always @ (button_left or button_right) begin
		if (button_left) begin
			delta_heading = 4'b1111;
		end 
		else begin 
			if (button_right) delta_heading = 4'b0001;
			else delta_heading = 4'b0000;
		end
	end

/* Currently Broken
	always @ (gameclock) begin
		if (buttons_were_pressed) delta_heading = 0;
		else begin
			if (button_left) begin
				delta_heading = 4'b0001;
				buttons_were_pressed = 1;
			end 
			else if (button_right) begin
				delta_heading = 4'b1111;
				buttons_were_pressed = 1;
			end
		end
		if (~button_left & ~button_right) buttons_were_pressed = 0;
	end
*/
endmodule

module delta_pos(change_x, change_y, heading, speeding_now);
	input 	[3:0] heading;
	input 	speeding_now;
	output 	[14:0] change_x, change_y;
	reg 	[14:0] change_x, change_y;

	initial begin
		change_x = 0;
		change_y = 0;
	end
	
	always @ (speeding_now or heading) begin
	case(speeding_now)
		'd0:begin
			case(heading)
				'd0:begin
					change_x <= 48;
					change_y <= 0;
					end
				'd1:begin
					change_x <= 44;
					change_y <= 18;
					end
				'd2:begin
					change_x <= 33;
					change_y <= 33;
					end
				'd3:begin
					change_x <= 18;
					change_y <= 44;
					end
				'd4:begin
					change_x <= 0;
					change_y <= 48;
					end
				'd5:begin
					change_x <= -18;
					change_y <= 44;
					end
				'd6:begin
					change_x <= -33;
					change_y <= 33;
					end
				'd7:begin
					change_x <= -44;
					change_y <= 18;
					end
				'd8:begin
					change_x <= -48;
					change_y <= 0;
					end
				'd9:begin
					change_x <= -44;
					change_y <= -18;
					end
				'd10:begin
					change_x <= -33;
					change_y <= -33;
					end
				'd11:begin
					change_x <= -18;
					change_y <= -44;
					end
				'd12:begin
					change_x <= 0;
					change_y <= -48;
					end
				'd13:begin
					change_x <= 18;
					change_y <= -44;
					end
				'd14:begin
					change_x <= 33;
					change_y <= -33;
					end
				'd15:begin
					change_x <= 44;
					change_y <= -18;
					end
			endcase
		end
		'd1:begin
			case(heading)
				'd0:begin
					change_x <= 96;
					change_y <= 0;
					end
				'd1:begin
					change_x <= 88;
					change_y <= 36;
					end
				'd2:begin
					change_x <= 67;
					change_y <= 67;
					end
				'd3:begin
					change_x <= 36;
					change_y <= 88;
					end
				'd4:begin
					change_x <= 0;
					change_y <= 96;
					end
				'd5:begin
					change_x <= -36;
					change_y <= 88;
					end
				'd6:begin
					change_x <= -67;
					change_y <= 67;
					end
				'd7:begin
					change_x <= -88;
					change_y <= 36;
					end
				'd8:begin
					change_x <= -96;
					change_y <= 0;
					end
				'd9:begin
					change_x <= -88;
					change_y <= -36;
					end
				'd10:begin
					change_x <= -67;
					change_y <= -67;
					end
				'd11:begin
					change_x <= -36;
					change_y <= -88;
					end
				'd12:begin
					change_x <= 0;
					change_y <= -96;
					end
				'd13:begin
					change_x <= 36;
					change_y <= -88;
					end
				'd14:begin
					change_x <= 67;
					change_y <= -67;
					end
				'd15:begin
					change_x <= 88;
					change_y <= -36;
					end
			endcase
		end
	endcase
	end
endmodule

module animal(posx, posy, heading, burst_enabled, type_out, delta_heading, heading_in, burst, type_in, posx_init, posy_init, gameclock);
	input 	gameclock;					// clock that defines speed of the game
	input 	[3:0] heading_in; 	// new heading value
	input 	[3:0] delta_heading; 	// two's compliment change to heading value
	input		burst;						// boolean controlling whether the animal is in speed mode
	input    [9:0]	posx_init, posy_init; // inital placement of animals
	input 	[3:0] type_in;				// Sent to type_out
	output	[3:0] type_out;			// Sent to graphics through world to tell what creature to draw
	output 	[9:0] posx;					// Sent to graphics through world to tell where to draw the creature
	output 	[9:0] posy;					// Sent to graphics through world to tell where to draw the creature
	output 	[3:0] heading; 			// Sent to graphics through world to tell in what orientation to draw the creature
												// 2's compliment; 'b0000 is right, positive counterclockwise rotation 
	output 	burst_enabled;				// Sent to world to tell whether burst is charged

	// Initialize registers for animal attributes
	reg [15:0] 	posx_reg;			// large enough for 640 x 480 monitor (up to 1024 max val), 4-bit precision in pixel location
	reg [15:0] 	posy_reg;			// large enough for 640 x 480 monitor (up to 1024 max val), 4-bit precision in pixel location

	reg [3:0] 	heading;		//	16 values
	reg [10:0] 	energy; 		// large enough for 10-second speed burst at 60Hz framerate decrementing each frame
	reg			burst_enabled;
	reg 			speeding;		

	wire [14:0] delta_x, delta_y;
	wire [3:0] heading_now;
	wire speeding_now;
	wire signed [11:0] posx_signed,posy_signed;
	
	assign heading_now = heading;
	assign speeding_now = speeding;
	
	// Initialize Methods
	delta_pos	delta_xy	(delta_x, delta_y, heading_now, speeding_now);

	// Initialize Values for Animal Attributes
	assign type_out = type_in;
	assign posx_signed = posx_reg[15:4];
	assign posy_signed = posy_reg[15:4];
	assign posx = posx_signed[10:0];
	assign posy = posy_signed[10:0];

	initial begin
		heading 			= 0;
		posx_reg 		= 320; //posx_init << 4; 	// Incl.ScreenRes.X/2
		posy_reg 		= 240; //posy_init << 4; 	// Incl.ScrrenRes.Y/2
		burst_enabled 	= 1;
		speeding 		= 0;
		energy 			= 60; 	// Incl.Energy*Incl.Framerate
	end
	
	always @ (posedge gameclock) begin
		if (burst_enabled & burst) begin
			burst_enabled = 0;
			speeding = 1;
			end

		if (speeding) energy = energy - 1;
		if (~speeding) energy = energy + 1;

		if (energy > 60) begin
			energy = 60;
			burst_enabled = 1;
			end

		if (energy == 0) begin
			speeding = 0;
			burst_enabled = 0;
			end

		if (heading_in == 15) begin
			heading = heading + delta_heading;
		end
		else begin
			heading = heading_in;
		end
		posx_reg = posx_reg + delta_x;
		posy_reg = posy_reg + delta_y;

		if (posx <= 19) posx_reg = 619 << 4;
		else if (posx >= 620) posx_reg  = 20 << 4;
		if (posy <= 19) posy_reg = 459 << 4;
		else if (posy >= 460) posy_reg = 20 << 4;
	end

endmodule
