module basic_alg( 
	   heading_out, burst_state, caught, animal_controlling,
		pos_x_in, pos_y_in, heading_in, burst_enabled
		);
		
	input [79:0]  pos_x_in, pos_y_in;
	input [31:0]  heading_in;
	input [7:0] burst_enabled;
	input [2:0] animal_controlling;
	
	output burst_state;
	output caught;
	output [3:0] heading_out;
	
	wire signed [9:0] x, x_3_4,x_1_2_bit,x_1_4_bit, y,y_3_4,y_1_2_bit,y_1_4_bit;
	wire [9:0] x_abs,y_abs;
	wire signed [8:0] x_1_2,y_1_2;
	wire signed [7:0] x_1_4,y_1_4;
	
	reg [3:0] heading_in_reg;
	reg high_bit_x_fracs,high_bit_y_fracs;
	reg signed [9:0] x_reg,y_reg;
	reg caught_reg;
	reg [3:0] heading_out_reg;
	
		
	abs abs1(x_abs,x);
	abs abs2(y_abs,y);
	
	assign burst_state = burst_enabled;
	
	assign x = x_reg;
	assign y = y_reg;
	
	
							//determine the value of 1/2 and 1/4 of x and y so that the coordinate plane can be broken into 8 segments
	assign x_1_2_bit = x >> 1;			//bit shift and then truncate(to make sure 2's complement numbers still work) to find the 1/2 and 1/4 values
	assign x_1_2_bit[9] = x_1_2_bit[8];
	assign x_1_4_bit = x >> 2;
	assign x_1_4_bit[8] = x_1_4_bit[7];
	assign x_1_4_bit[9] = x_1_4_bit[7];
	assign x_1_2 = x_1_2_bit;
	assign x_1_4 = x_1_4_bit;
	assign x_3_4 = x_1_2 + x_1_4;
	
	assign y_1_2_bit = y >> 1;
	assign y_1_2_bit[9] = y_1_2_bit[8];
	assign y_1_4_bit = y >> 2;
	assign y_1_4_bit[8] = y_1_4_bit[7];
	assign y_1_4_bit[9] = y_1_4_bit[7];
	assign y_1_2 = y_1_2_bit;
	assign y_1_4 = y_1_4_bit;
	assign y_3_4 = y_1_2 + y_1_4;
	
	
	
	always @ (*) begin
		case (animal_controlling)			//determine which cat we are controlling, then find the relative position of the mouse
			1:
			begin
				x_reg = pos_x_in[9:0] - pos_x_in[19:10];
				y_reg = (pos_y_in[19:10] - pos_y_in[9:0]);
				heading_in_reg = heading_in[7:4];
			end
			2:
			begin
				x_reg = pos_x_in[9:0] - pos_x_in[29:20];
				y_reg = (pos_y_in[29:20] - pos_y_in[9:0]);
				heading_in_reg = heading_in[11:8];
			end
			3:
			begin
				x_reg = pos_x_in[9:0] - pos_x_in[39:30];
				y_reg = (pos_y_in[39:30] - pos_y_in[9:0]);
				heading_in_reg = heading_in[15:12];
			end
			4:
			begin
				x_reg = pos_x_in[9:0] - pos_x_in[49:40];
				y_reg = (pos_y_in[49:40] - pos_y_in[9:0]);
				heading_in_reg = heading_in[19:16];
			end
			5:
			begin
				x_reg = pos_x_in[9:0] - pos_x_in[59:50];
				y_reg = (pos_y_in[59:50] - pos_y_in[9:0]);
				heading_in_reg = heading_in[23:20];
			end
			6:
			begin
				x_reg = pos_x_in[9:0] - pos_x_in[69:60];
				y_reg = (pos_y_in[69:60] - pos_y_in[9:0]);
				heading_in_reg = heading_in[27:24];
			end
			7:
			begin
				x_reg = pos_x_in[9:0] - pos_x_in[79:70];
				y_reg = (pos_y_in[79:70] - pos_y_in[9:0]);
				heading_in_reg = heading_in[31:28];
			end
		
		endcase
	end
	
	always @(*) begin 
		
		//8ths granularity (number of directions cat can head)
		if ( y >= -x_1_2 && y < x_1_2 ) begin	//estimated 1/16's of the circle--each region makes determines the heading_out (positive x-axis is heading_out 0)
			heading_out_reg = 0;
		end
		if ( y >= x_1_2 && x > y_1_2 ) begin 
			heading_out_reg = 14;
		end
		if ( x <= y_1_2 && x > -y_1_2 ) begin 
			heading_out_reg = 12;
		end
		if ( x <= -y_1_2 && y > -x_1_2 ) begin 
			heading_out_reg = 10;
		end
		if ( y <= -x_1_2 && y > x_1_2 ) begin 
			heading_out_reg = 8;
		end
		if ( y <= x_1_2 && x < y_1_2 ) begin 
			heading_out_reg = 6;
		end
		if ( x >= y_1_2 && x < -y_1_2 ) begin 
			heading_out_reg = 4;
		end
		if ( x >= -y_1_2 && y < -x_1_2 ) begin 
			heading_out_reg = 2;
		end		
		
		
		//16ths granularity, not in affect
		/*if ( y >= -x_1_4 && y < x_1_4 ) begin	//estimated 1/32's of the circle--each region makes determines the heading (positive x-axis is heading 0)
			heading = 0;
		end
		if ( y >= x_1_4 && y < x_3_4 ) begin 
			heading = 1;
		end
		if ( y >= x_3_4 && x > y_3_4 ) begin 
			heading = 2;
		end
		if ( x <= y_3_4 && x > y_1_4 ) begin 
			heading = 3;
		end
		if ( x <= y_1_4 && x > -y_1_4 ) begin 
			heading = 4;
		end
		if ( x <= -y_1_4 && x > -y_3_4 ) begin 
			heading = 5;
		end
		if ( x <= -y_3_4 && y > -x_3_4 ) begin 
			heading = 6;
		end
		if ( y <= -x_3_4 && y > -x_1_4 ) begin 
			heading = 7;
		end
		if ( y <= -x_1_4 && y > x_1_4 ) begin 
			heading = 8;
		end
		if ( y <= x_1_4 && y > x_3_4 ) begin 
			heading = 9;
		end
		if ( y <= x_3_4 && x < y_3_4 ) begin 
			heading = 10;
		end
		if ( x >= y_3_4 && x < y_1_4 ) begin 
			heading = 11;
		end
		if ( x >= y_1_4 && x < -y_1_4 ) begin 
			heading = 12;
		end
		if ( x >= -y_1_4 && x < -y_3_4 ) begin 
			heading = 13;
		end
		if ( x >= -y_3_4 && y < -x_3_4 ) begin 
			heading = 14;
		end
		if ( y >= -x_3_4 && y < -x_1_4 ) begin 
			heading = 15;
		end
		*/
		
		if ( y_abs < 10 && x_abs < 10 ) begin            //if the cat has caught the mouse, put the flag high
			heading_out_reg = heading_in_reg;
			caught_reg = 1;
		end
		else begin
			caught_reg = 0;
		end

	end
	assign caught = caught_reg;
	assign heading_out = heading_out_reg;
	
endmodule

module abs(out,in);                    //absolute value function
	output [9:0] out;
	input [9:0] in;
	
	reg [9:0] out_reg;
	
	always @ (*) begin
		if (in <=0) begin
			out_reg = -in;
		end
		else begin
			out_reg = in;
		end
	end
	
	assign out = out_reg;
endmodule