module RECTANGLES
(
posx,
posy,
heading,
pixel_clock,
reset,
pixel_count0,
line_count0,
CAUGHT,
vga_red_data,
vga_green_data,
vga_blue_data
);

input				pixel_clock;
input				reset;
input [10:0] 	pixel_count0;
input [9:0]		line_count0;
input CAUGHT;
input [79:0] posx; 					// 8 separate 10-bit x-positions, sequentially
input [79:0] posy; 					// 8 separate 10-bit y-positions, sequentially
input [31:0] heading; 				// 8 separate 4-bit headings, sequentially

output			vga_red_data;
output			vga_green_data;
output			vga_blue_data;

reg [10:0]		pixel_count;
reg [9:0]		line_count;

reg				red_data;
reg				green_data;
reg				blue_data;

wire [2:0]		write_data = {red_data, green_data, blue_data};
wire [2:0]		read_data;
assign {vga_red_data, vga_green_data, vga_blue_data} = read_data;

// Create the color bars in memory, alternating memory locations every other
// line to allow for computation time.

// Only the least significant 10 bits are required out of the total 11 bits
// of pixel count, because active screen area has a maximum width of 1024.

// The remaining bits of pixel count are used to keep track of the front porch,
// synch, and back porch, which make up the blank period. It is OK that this
// module will output color data during the blank period, because the VIDEO_OUT
// module overrides the output of this module with zeros when the blank signal
// is high.
wire [10:0]		write_addr = {!line_count0[0], pixel_count0[9:0]};
wire [10:0]		read_addr = {line_count0[0], pixel_count0[9:0]};
wire				write_enable = 1'b1; // write every clock posedge


// compensate for the delay caused by displaying the previous line while
// calculating the current line by using a line counter that is one line
// ahead of the original line counter
always @ (posedge pixel_clock) begin
	if ((line_count0 == (`V_TOTAL - 2)) & (pixel_count0 == (`H_TOTAL - 3)))
		// last pixel in last line of frame, so reset line counter
		line_count <= 10'h000;
	
	else if (pixel_count0 == (`H_TOTAL - 3))
		// last pixel but not last line, so increment line counter
		line_count <= line_count + 1;
end

// compensate for putting the pixel_count generation and the code that
// checks it on the same clock edge
always @ (posedge pixel_clock) begin
	if ((pixel_count0 == (`H_TOTAL - 3)))
		// last pixel in last line of frame, so reset line counter
		pixel_count <= 10'h000;
	
	else
		// last pixel but not last line, so increment line counter
		pixel_count <= pixel_count + 1;
end


////////////////////////////////////////////////////////////////////////////

//reg [9:0] xposition1, yposition1;
reg [9:0] xlower1, xupper1, ylower1, yupper1;

//xposition1 = posx[9:0] ;
//yposition1 = posy[9:0] ;

//reg [9:0] xposition2, yposition2;
reg [9:0] xlower2, xupper2, ylower2, yupper2;

reg [9:0] xlowercat1, xuppercat1, ylowercat1, yuppercat1;

reg [9:0] xlowermouse1, xuppermouse1, ylowermouse1, yuppermouse1;
reg [3:0] heading1, heading2, headingcat1, headingmouse1;
//xposition2 = posx[19:10] ;
//yposition2 = posy[19:10] ;


/*initial
	begin
		xposition = 330;
		yposition = 200;
	end
*/	

/*
always @ (posedge pixel_clock) begin
	assign xlower1 = xposition1 - 8;
	assign xupper1 = xposition1 + 8;
	assign ylower1 = yposition1 + 8;
	assign yupper1 = yposition1 - 8;

	assign xlower2 = xposition2 - 8;
	assign xupper2 = xposition2 + 8;
	assign ylower2 = yposition2 + 8;
	assign yupper2 = yposition2 - 8;

end
*/

// Define a small rectangle around the animal
// where a picture will be read from memory.
always @ (posedge pixel_clock) begin
	assign xlower1 = posx[39:30] - 8;
	assign xupper1 = posx[39:30] + 8;
	assign ylower1 = posy[39:30] + 8;
	assign yupper1 = posy[39:30] - 8;
	assign heading1 = heading[15:12];

	assign xlower2 = posx[29:20] - 8;
	assign xupper2 = posx[29:20] + 8;
	assign ylower2 = posy[29:20] + 8;
	assign yupper2 = posy[29:20] - 8;
	assign heading2 = heading[11:8];

	assign xlowercat1 = posx[19:10] - 16;
	assign xuppercat1 = posx[19:10] + 16;
	assign ylowercat1 = posy[19:10] + 11;
	assign yuppercat1 = posy[19:10] - 11;
	assign headingcat1 = heading[7:4];
	
	assign xlowermouse1 = posx[9:0] - 13;
	assign xuppermouse1 = posx[9:0] + 13;
	assign ylowermouse1 = posy[9:0] + 9;
	assign yuppermouse1 = posy[9:0] - 9;
	assign headingmouse1 = heading[3:0];
	
end

///////////////////////////////////////////////////

// Define pictures.

reg [15:0] shape1memR[15:0];
reg [15:0] shape1memG[15:0];
reg [15:0] shape1memB[15:0];
reg [31:0] shaperowR, shaperowG, shaperowB;  // This needs to be as wide (in x) as the widest shape

reg [15:0] shape2memR[15:0];
reg [15:0] shape2memG[15:0];
reg [15:0] shape2memB[15:0];

reg [31:0] catshape1R[21:0];
reg [31:0] catshape1G[21:0];
reg [31:0] catshape1B[21:0];

reg [25:0] mouseshape1R[18:0];
reg [25:0] mouseshape1G[18:0];
reg [25:0] mouseshape1B[18:0];

initial begin
	// Define shape 1.
	shape1memR[0] = 16'b1111111111111111;
	shape1memR[1] = 16'b1111111111111111;
	shape1memR[2] = 16'b1111111111111111;
	shape1memR[3] = 16'b1111111111111111;
	shape1memR[4] = 16'b1111111111111111;
	shape1memR[5] = 16'b1111111111111111;
	shape1memR[6] = 16'b1111111111111111;
	shape1memR[7] = 16'b1111111111111111;
	shape1memR[8] = 16'b1111111111111111;
	shape1memR[9] = 16'b1111111111111111;
	shape1memR[10] = 16'b1111111111111111;
	shape1memR[11] = 16'b1111111111111111;
	shape1memR[12] = 16'b1111111111111111;
	shape1memR[13] = 16'b1111111111111111;
	shape1memR[14] = 16'b1111111111111111;
	shape1memR[15] = 16'b1111111111111111;

	shape1memG[0] = 16'b0000000000000000;
	shape1memG[1] = 16'b0000000000000000;
	shape1memG[2] = 16'b0000000000000000;
	shape1memG[3] = 16'b0000000000000000;
	shape1memG[4] = 16'b0000000000000000;
	shape1memG[5] = 16'b0000000000000000;
	shape1memG[6] = 16'b0000000000000000;
	shape1memG[7] = 16'b0000000000000000;
	shape1memG[8] = 16'b0000000000000000;
	shape1memG[9] = 16'b0000000000000000;
	shape1memG[10] = 16'b0000000000000000;
	shape1memG[11] = 16'b0000000000000000;
	shape1memG[12] = 16'b0000000000000000;
	shape1memG[13] = 16'b0000000000000000;
	shape1memG[14] = 16'b0000000000000000;
	shape1memG[15] = 16'b0000000000000000;

	shape1memB[0] = 16'b1111111111111111;
	shape1memB[1] = 16'b1111111111111111;
	shape1memB[2] = 16'b1111111111111111;
	shape1memB[3] = 16'b1111111111111111;
	shape1memB[4] = 16'b1111111111111111;
	shape1memB[5] = 16'b1111111111111111;
	shape1memB[6] = 16'b0000000000000000;
	shape1memB[7] = 16'b0000000000000000;
	shape1memB[8] = 16'b0000000000000000;
	shape1memB[9] = 16'b0000000000000000;
	shape1memB[10] = 16'b0000000000000000;
	shape1memB[11] = 16'b1111111111111111;
	shape1memB[12] = 16'b1111111111111111;
	shape1memB[13] = 16'b1111111111111111;
	shape1memB[14] = 16'b1111111111111111;
	shape1memB[15] = 16'b1111111111111111;

	// Define shape 2.
	shape2memR[0] 	= 16'b0000111111110000;
	shape2memR[1] 	= 16'b0011111111111100;
	shape2memR[2] 	= 16'b0111111111111110;
	shape2memR[3] 	= 16'b0111111111111110;
	shape2memR[4] 	= 16'b1111001111001111;
	shape2memR[5] 	= 16'b1111111111111111;
	shape2memR[6] 	= 16'b1111111111111111;
	shape2memR[7] 	= 16'b1111111111111111;
	shape2memR[8] 	= 16'b1111111111111111;
	shape2memR[9] 	= 16'b1111111111111111;
	shape2memR[10] = 16'b1111111111111111;
	shape2memR[11] = 16'b1110011111001111;
	shape2memR[12] = 16'b1111001110011111;
	shape2memR[13] = 16'b0111100000111110;
	shape2memR[14] = 16'b0011111111111100;
	shape2memR[15] = 16'b0000111111110000;

	shape2memG[0] 	= 16'b0000111111110000;
	shape2memG[1] 	= 16'b0011111111111100;
	shape2memG[2] 	= 16'b0111111111111110;
	shape2memG[3] 	= 16'b0111111111111110;
	shape2memG[4] 	= 16'b1111001111001111;
	shape2memG[5] 	= 16'b1111111111111111;
	shape2memG[6] 	= 16'b1111111111111111;
	shape2memG[7] 	= 16'b1111111111111111;
	shape2memG[8] 	= 16'b1111111111111111;
	shape2memG[9] 	= 16'b1111111111111111;
	shape2memG[10] = 16'b1111111111111111;
	shape2memG[11] = 16'b1110011111001111;
	shape2memG[12] = 16'b1111001110011111;
	shape2memG[13] = 16'b0111100000111110;
	shape2memG[14] = 16'b0011111111111100;
	shape2memG[15] = 16'b0000111111110000;

	shape2memB[0] = 16'b0000000000000000;
	shape2memB[1] = 16'b0000000000000000;
	shape2memB[2] = 16'b0000000000000000;
	shape2memB[3] = 16'b0000000000000000;
	shape2memB[4] = 16'b0000000000000000;
	shape2memB[5] = 16'b0000000000000000;
	shape2memB[6] = 16'b0000000000000000;
	shape2memB[7] = 16'b0000000000000000;
	shape2memB[8] = 16'b0000000000000000;
	shape2memB[9] = 16'b0000000000000000;
	shape2memB[10] = 16'b0000000000000000;
	shape2memB[11] = 16'b0000000000000000;
	shape2memB[12] = 16'b0000000000000000;
	shape2memB[13] = 16'b0000000000000000;
	shape2memB[14] = 16'b0000000000000000;
	shape2memB[15] = 16'b0000000000000000;
	
	// Cat!

	catshape1R[0] = 32'b11000000000000000000000000000000;
	catshape1R[1] = 32'b00100000000000000000000000000000;
	catshape1R[2] = 32'b00100000000000000000000000000000;
	catshape1R[3] = 32'b00010000000000000000000000000000;
	catshape1R[4] = 32'b00010000000000000000000000000000;
	catshape1R[5] = 32'b00010000000000000000000000010000;
	catshape1R[6] = 32'b00010000000000000000000000111000;
	catshape1R[7] = 32'b00010000000000000000000000111110;
	catshape1R[8] = 32'b00010000000000000000000001111111;
	catshape1R[9] = 32'b00001000111111111111111101111111;
	catshape1R[10] = 32'b00001011111111111111111111111110;
	catshape1R[11] = 32'b00001111111111111111111111111100;
	catshape1R[12] = 32'b00000111111111111111111111100000;
	catshape1R[13] = 32'b00000111111111111111111111100000;
	catshape1R[14] = 32'b00000011111111111111111111000000;
	catshape1R[15] = 32'b00000000111111111111111110000000;
	catshape1R[16] = 32'b00000000111111111111111100000000;
	catshape1R[17] = 32'b00000000101000000000010100000000;
	catshape1R[18] = 32'b00000001101100000000110110000000;
	catshape1R[19] = 32'b00000001000100000000100010000000;
	catshape1R[20] = 32'b00000001000100000000100010000000;
	catshape1R[21] = 32'b00000001000100000000100010000000;

	catshape1G[0] = 32'b00000000000000000000000000000000;
	catshape1G[1] = 32'b00000000000000000000000000000000;
	catshape1G[2] = 32'b00000000000000000000000000000000;
	catshape1G[3] = 32'b00000000000000000000000000000000;
	catshape1G[4] = 32'b00000000000000000000000000000000;
	catshape1G[5] = 32'b00000000000000000000000000000000;
	catshape1G[6] = 32'b00000000000000000000000000000000;
	catshape1G[7] = 32'b00000000000000000000000000000000;
	catshape1G[8] = 32'b00000000000000000000000000000000;
	catshape1G[9] = 32'b00000000000000000000000000000000;
	catshape1G[10] = 32'b00000000000000000000000000000000;
	catshape1G[11] = 32'b00000000000000000000000000000000;
	catshape1G[12] = 32'b00000000000000000000000000000000;
	catshape1G[13] = 32'b00000000000000000000000000000000;
	catshape1G[14] = 32'b00000000000000000000000000000000;
	catshape1G[15] = 32'b00000000000000000000000000000000;
	catshape1G[16] = 32'b00000000000000000000000000000000;
	catshape1G[17] = 32'b00000000000000000000000000000000;
	catshape1G[18] = 32'b00000000000000000000000000000000;
	catshape1G[19] = 32'b00000000000000000000000000000000;
	catshape1G[20] = 32'b00000000000000000000000000000000;
	catshape1G[21] = 32'b00000000000000000000000000000000;

	catshape1B[0] = 32'b00000000000000000000000000000000;
	catshape1B[1] = 32'b00000000000000000000000000000000;
	catshape1B[2] = 32'b00000000000000000000000000000000;
	catshape1B[3] = 32'b00000000000000000000000000000000;
	catshape1B[4] = 32'b00000000000000000000000000000000;
	catshape1B[5] = 32'b00000000000000000000000000000000;
	catshape1B[6] = 32'b00000000000000000000000000000000;
	catshape1B[7] = 32'b00000000000000000000000000000000;
	catshape1B[8] = 32'b00000000000000000000000000000000;
	catshape1B[9] = 32'b00000000000000000000000000000000;
	catshape1B[10] = 32'b00000000000000000000000000000000;
	catshape1B[11] = 32'b00000000000000000000000000000000;
	catshape1B[12] = 32'b00000000000000000000000000000000;
	catshape1B[13] = 32'b00000000000000000000000000000000;
	catshape1B[14] = 32'b00000000000000000000000000000000;
	catshape1B[15] = 32'b00000000000000000000000000000000;
	catshape1B[16] = 32'b00000000000000000000000000000000;
	catshape1B[17] = 32'b00000000000000000000000000000000;
	catshape1B[18] = 32'b00000000000000000000000000000000;
	catshape1B[19] = 32'b00000000000000000000000000000000;
	catshape1B[20] = 32'b00000000000000000000000000000000;
	catshape1B[21] = 32'b00000000000000000000000000000000;

	// Mouse!
	
	mouseshape1R[0] = 26'b00000000000000000000000000;
	mouseshape1R[1] = 26'b00000000000000000000000000;
	mouseshape1R[2] = 26'b00000000000000000000000000;
	mouseshape1R[3] = 26'b00000000000000000000000000;
	mouseshape1R[4] = 26'b00000000000000000000000000;
	mouseshape1R[5] = 26'b00000000000000000000000000;
	mouseshape1R[6] = 26'b00000000000000000000000000;
	mouseshape1R[7] = 26'b00000000000000000000000000;
	mouseshape1R[8] = 26'b00000000000000000000000000;
	mouseshape1R[9] = 26'b00000000000000000000000000;
	mouseshape1R[10] = 26'b00000000000000000000000000;
	mouseshape1R[11] = 26'b00000000000000000000000000;
	mouseshape1R[12] = 26'b00000000000000000000000000;
	mouseshape1R[13] = 26'b00000000000000000000000000;
	mouseshape1R[14] = 26'b00000000000000000000000000;
	mouseshape1R[15] = 26'b00000000000000000000000000;
	mouseshape1R[16] = 26'b00000000000000000000000000;
	mouseshape1R[17] = 26'b00000000000000000000000000;
	mouseshape1R[18] = 26'b00000000000000000000000000;

	mouseshape1G[0] = 26'b00000000000000000000010000;
	mouseshape1G[1] = 26'b00000000000000000000001000;
	mouseshape1G[2] = 26'b00000001100000000000001000;
	mouseshape1G[3] = 26'b00000011110000000000000100;
	mouseshape1G[4] = 26'b00000011110000000000000100;
	mouseshape1G[5] = 26'b00000011110000000000000010;
	mouseshape1G[6] = 26'b00011111100000000000000010;
	mouseshape1G[7] = 26'b01111111111111111110000010;
	mouseshape1G[8] = 26'b11111111111111111111100001;
	mouseshape1G[9] = 26'b11111111111111111111110010;
	mouseshape1G[10] = 26'b11111111111111111111110010;
	mouseshape1G[11] = 26'b01111111111111111111111100;
	mouseshape1G[12] = 26'b00000011111111111111110000;
	mouseshape1G[13] = 26'b00000011111111111111110000;
	mouseshape1G[14] = 26'b00000001111111111111100000;
	mouseshape1G[15] = 26'b00000000011111111110000000;
	mouseshape1G[16] = 26'b00000000011000000011000000;
	mouseshape1G[17] = 26'b00000000100100000100100000;
	mouseshape1G[18] = 26'b00000001000010001000010000;

	mouseshape1B[0] = 26'b00000000000000000000010000;
	mouseshape1B[1] = 26'b00000000000000000000001000;
	mouseshape1B[2] = 26'b00000001100000000000001000;
	mouseshape1B[3] = 26'b00000011110000000000000100;
	mouseshape1B[4] = 26'b00000011110000000000000100;
	mouseshape1B[5] = 26'b00000011110000000000000010;
	mouseshape1B[6] = 26'b00011111100000000000000010;
	mouseshape1B[7] = 26'b01111111111111111110000010;
	mouseshape1B[8] = 26'b11111111111111111111100001;
	mouseshape1B[9] = 26'b11111111111111111111110010;
	mouseshape1B[10] = 26'b11111111111111111111110010;
	mouseshape1B[11] = 26'b01111111111111111111111100;
	mouseshape1B[12] = 26'b00000011111111111111110000;
	mouseshape1B[13] = 26'b00000011111111111111110000;
	mouseshape1B[14] = 26'b00000001111111111111100000;
	mouseshape1B[15] = 26'b00000000011111111110000000;
	mouseshape1B[16] = 26'b00000000011000000011000000;
	mouseshape1B[17] = 26'b00000000100100000100100000;
	mouseshape1B[18] = 26'b00000001000010001000010000;

end

////////////////////////////////////////////////////


//reg UpdatePositions;
//always @(posedge pixel_clock) begin
//	UpdatePositions <= ((line_count==200) & (pixel_count==0)); //update once a frame
//end
//always @(posedge UpdatePositions) begin


// Within the rectangle defined above, read the 
// picture from memory.
always @(posedge pixel_clock) begin
/*	if ((pixel_count >= xlower1) & (pixel_count < xupper1) & (line_count < ylower1) & (line_count >= yupper1)) begin
		//red_data = 1'b1;
		//green_data = 1'b0;
		//blue_data = 1'b1;
		shaperowR = shape1memR[line_count - yupper1 ];
		red_data = shaperowR[pixel_count - xlower1 ];
		shaperowG = shape1memG[line_count - yupper1 ];
		green_data = shaperowG[pixel_count - xlower1 ];
		shaperowB = shape1memB[line_count - yupper1 ];
		blue_data = shaperowB[pixel_count - xlower1 ];
	
	end else if ((pixel_count >= xlower2) & (pixel_count < xupper2) & (line_count < ylower2) & (line_count >= yupper2)) begin
		//red_data = 1'b0;
		//green_data = 1'b0;
		//blue_data = 1'b1;
		shaperowR = shape2memR[line_count - yupper2 ];
		red_data = shaperowR[pixel_count - xlower2  ];
		shaperowG = shape2memG[line_count - yupper2  ];
		green_data = shaperowG[pixel_count - xlower2 ];
		shaperowB = shape2memB[line_count - yupper2  ];
		blue_data = shaperowB[pixel_count - xlower2  ];
*/
	//end else 
	if ((headingcat1 >= 5) & (headingcat1 <= 12) & (pixel_count >= xlowercat1) & (pixel_count < xuppercat1) & (line_count < ylowercat1) & (line_count >= yuppercat1)) begin
		shaperowR = catshape1R[line_count - yuppercat1 ];
		red_data = shaperowR[pixel_count - xlowercat1  ];
		shaperowG = catshape1G[line_count - yuppercat1  ];
		green_data = shaperowG[pixel_count - xlowercat1 ];
		shaperowB = catshape1B[line_count - yuppercat1  ];
		blue_data = shaperowB[pixel_count - xlowercat1  ];
		
	end else if ((headingcat1 <= 4) & (headingcat1 >= 0) & (pixel_count >= xlower2) & (pixel_count < xupper2) & (line_count < ylower2) & (line_count >= yupper2)) begin
	 	// if the cat has a heading less than 4, it will change to a smiley face (for now)
		//red_data = 1'b0;
		//green_data = 1'b0;
		//blue_data = 1'b1;
		shaperowR = shape2memR[line_count - yupper2 ];
		red_data = shaperowR[pixel_count - xlower2  ];
		shaperowG = shape2memG[line_count - yupper2  ];
		green_data = shaperowG[pixel_count - xlower2 ];
		shaperowB = shape2memB[line_count - yupper2  ];
		blue_data = shaperowB[pixel_count - xlower2  ];
		
	end else if ((headingcat1 >= 13) & (headingcat1 <= 15) & (pixel_count >= xlower2) & (pixel_count < xupper2) & (line_count < ylower2) & (line_count >= yupper2)) begin
	 	// if the cat has a heading greater than 12, it will change to a smiley face (for now)
		//red_data = 1'b0;
		//green_data = 1'b0;
		//blue_data = 1'b1;
		shaperowR = shape2memR[line_count - yupper2 ];
		red_data = shaperowR[pixel_count - xlower2  ];
		shaperowG = shape2memG[line_count - yupper2  ];
		green_data = shaperowG[pixel_count - xlower2 ];
		shaperowB = shape2memB[line_count - yupper2  ];
		blue_data = shaperowB[pixel_count - xlower2  ];
		
	end else if ((pixel_count >= xlowermouse1) & (pixel_count < xuppermouse1) & (line_count < ylowermouse1) & (line_count >= yuppermouse1)) begin
		shaperowR = mouseshape1R[line_count - yuppermouse1 ];
		red_data = shaperowR[pixel_count - xlowermouse1  ];
		shaperowG = mouseshape1G[line_count - yuppermouse1  ];
		green_data = shaperowG[pixel_count - xlowermouse1 ];
		shaperowB = mouseshape1B[line_count - yuppermouse1  ];
		blue_data = shaperowB[pixel_count - xlowermouse1  ];	
	end else begin
		red_data = 1'b0;
		green_data = 1'b0;
		blue_data = 1'b0;
	end
	
end


////////////////////////////////////////////////////////////////////////////////////////



// Instantiate the video RAM
VIDEO_RAM VIDEO_RAM
(
pixel_clock,		// write clock
write_enable,		// write eanble
write_addr,			// write address
write_data,			// write data

pixel_clock,		// read clock
read_addr,			// read address
read_data			// read data	
);

endmodule //COLOR_BARS --> RECTANGLES