//  Code borrowed from Nate Smith and Eric Gallimore
//  Modified by Angela Sharer
`timescale 1ns / 1ps
module graphics(
VGA_HSYNCH,
VGA_VSYNCH,
VGA_OUT_RED,
VGA_OUT_GREEN,
VGA_OUT_BLUE,

POSX,
POSY,
HEADING,
//TYPE,
CAUGHT,
//GAME_CLOCK,
SYSTEM_CLOCK
);

input		 	 SYSTEM_CLOCK;					// 100MHz LVTTL SYSTEM CLOCK; ours is actually 50MHz; code works fine
//input 		 GAME_CLOCK;					// Currently does nothing.
input [79:0] POSX; 							// 8 separate 10-bit x-positions, listed sequentially
input [79:0] POSY; 							// 8 separate 10-bit y-positions, listed sequentially
input [31:0] HEADING; 						// 8 separate 4-bit headings, listed sequentially
//input [31:0] TYPE; 						// 8 separate 4-bit types, listed sequentially
input CAUGHT;


//output led1, led2;

output 			VGA_HSYNCH;					// horizontal sync for the VGA output connector
output			VGA_VSYNCH;					// vertical sync for the VGA output connector
output		 	VGA_OUT_RED;				// RED DAC data
output		 	VGA_OUT_GREEN;				// GREEN DAC data
output		 	VGA_OUT_BLUE;				// BLUE DAC data

wire				system_clock_buffered;	// buffered SYSTEM CLOCK
wire				pixel_clock;				// generated from SYSTEM CLOCK
wire				reset;						// reset asserted when DCMs are NOT LOCKED

wire				vga_red_data;				// red video data
wire				vga_green_data;			// green video data
wire				vga_blue_data;				// blue video data

// internal video timing signals
wire 				h_synch;
wire 				v_synch;
wire 				blank;
wire [10:0]		pixel_count;				// bit mapped pixel position within the line
wire [9:0]		line_count;					// bit mapped line number in a frame lines within the frame


//////////////////
//for testing: check to see if this is getting posx and posy correctly
/*
reg led1, led2;

always @(posedge SYSTEM_CLOCK) begin
	if (POSX[9:0] == 200) begin
	assign led1 = 1;
	end else begin
	assign led2 = 1;
	end
end
*/
////////////////////

// instantiate the clock generation module
CLOCK_GEN CLOCK_GEN 
(
SYSTEM_CLOCK,
system_clock_buffered,
pixel_clock,
reset
);

// instantiate the video timing generator
SVGA_TIMING_GENERATION SVGA_TIMING_GENERATION
(
pixel_clock,
reset,
h_synch,
v_synch,
blank,
pixel_count,
line_count
);


// instantiate the picture generator ("RECTANGLES")

RECTANGLES RECTANGLES
(
POSX,
POSY,
HEADING,
pixel_clock,
reset,
pixel_count,
line_count,
CAUGHT,
vga_red_data,
vga_green_data,
vga_blue_data
);

// instantiate the video output mux
VIDEO_OUT VIDEO_OUT
(
pixel_clock,
reset,
vga_red_data,
vga_green_data,
vga_blue_data,
h_synch,
v_synch,
blank,

VGA_HSYNCH,
VGA_VSYNCH,
VGA_OUT_RED,
VGA_OUT_GREEN,
VGA_OUT_BLUE
);

endmodule // MAIN --> Graphics